# Configuration file for the Sphinx documentation builder.
#
# This file only contains a selection of the most common options. For a full
# list see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
# import os
# import sys
# sys.path.insert(0, os.path.abspath('../src'))


# -- Project information -----------------------------------------------------
import xsar

project = 'xsar'
copyright = '2021, Ifremer LOPS/SIAM'
author = 'Olivier Archer'
version = xsar.xsar.__version__

# -- General configuration ---------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    'sphinx.ext.autodoc',
    'sphinx.ext.napoleon',
    'sphinx_rtd_theme',
    'nbsphinx',
    'jupyter_sphinx',
    'sphinxcontrib.programoutput'
]

# order by source
autodoc_member_order = 'bysource'

# Napoleon settings
napoleon_google_docstring = True
napoleon_numpy_docstring = True
napoleon_include_init_with_doc = False
napoleon_include_private_with_doc = False
napoleon_include_special_with_doc = True
napoleon_use_admonition_for_examples = False
napoleon_use_admonition_for_notes = False
napoleon_use_admonition_for_references = False
napoleon_use_ivar = False
napoleon_use_param = True
napoleon_use_rtype = True
napoleon_type_aliases = None


# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = ['_build', 'Thumbs.db', '.DS_Store']


# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
#html_theme = 'classic'
html_theme = 'sphinx_rtd_theme'

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ['_static']

html_style = 'css/xsar.css'

#html_logo = "_static/logo.png"
html_theme_options = {
    'logo_only': False,
    'display_version': True,
    'navigation_depth': 4,  # FIXME: doesn't work as expeted: should expand side menu
    'collapse_navigation': False # FIXME: same as above
}

# If true, links to the reST sources are added to the pages.
html_show_sourcelink = False

nbsphinx_allow_errors = False

nbsphinx_execute = 'always'

nbsphinx_timeout = 300

nbsphinx_prolog = """
Download this notebook from github_.

.. _github: https://github.com/oarcher/xsar/tree/develop/docs/{{ env.doc2path(env.docname, base=False) }}

----
"""

today_fmt = '%b %d %Y at %H:%M'


import xarray as xr
import xsar
import rasterio
import warnings
import os

class XsarXarrayBackend(xr.backends.common.BackendEntrypoint):
    def open_dataset(self,
                     dataset_id, resolution=None, resampling=rasterio.enums.Resampling.rms,
                     luts=False, dtypes=None, drop_variables=[]):
        ds = xsar.open_dataset(dataset_id, resolution=resolution, resampling=resampling, luts=luts, dtypes=dtypes)
        if not list(ds.chunks):
            warnings.warn('Not using `chunks` kw is discouraged when openning SAFE')
        return ds.drop_vars(drop_variables, errors='ignore')

    def guess_can_open(self, filename_or_obj):
        if isinstance(filename_or_obj, xsar.Sentinel1Meta):
            return True
        if isinstance(filename_or_obj, str) and os.path.basename(filename_or_obj).endswith('.SAFE'):
            return True
        return False



from xsar.raster_readers import _to_lon180
import pytest
import xarray as xr
import copy
import numpy as np

val_lon_180s = [[120.,121.,122.,123.],[120.6,121.7,122.8,123.9]]
lat = xr.DataArray(np.array([[40.,41.,42.,43.],[40.5,41.5,42.5,43.5]]), dims=('y','x'),
                   coords={'x':[1,2,3,4], 'y':[5,6]})
lon = xr.DataArray(np.array(val_lon_180s), dims=('y','x'),
                   coords={'x':[1,2,3,4], 'y':[5,6]})
lon_acheval = xr.DataArray(np.array([[179.,179.2,179.5,179.9],[-179.4,-179.7,-178.8,-179.1]]), dims=('y','x'),
                   coords={'x':[1,2,3,4], 'y':[5,6]})
lon_0_360 = xr.DataArray(np.array(val_lon_180s)+200., dims=('y','x'),
                   coords={'x':[1,2,3,4], 'y':[5,6]})
lon_0_360_treated = xr.DataArray(np.array(val_lon_180s)+200.-360., dims=('y','x'),
                   coords={'x':[1,2,3,4], 'y':[5,6]})
# latwithNan = copy.copy(lat)
ds = xr.Dataset()
ds['lon'] = lon
ds['lat'] = lat

ds_on_antimeridian = xr.Dataset()
ds_on_antimeridian['lon'] = lon_acheval
ds_on_antimeridian['lat'] = lat

ds_0_360 = xr.Dataset()
ds_0_360['lon'] = lon_0_360
ds_0_360['lat'] = lat

ds_0_360_expected = xr.Dataset()
ds_0_360_expected['lon'] = lon_0_360_treated
ds_0_360_expected['lat'] = lat
@pytest.mark.parametrize(
    ["ds", "expected"],
    (
        pytest.param(ds, ds, id="180_180"),
        pytest.param(ds_0_360, ds_on_antimeridian, id="0_360"),
        pytest.param(ds_on_antimeridian, ds_0_360_expected, id="180_180_a_cheval"),
    ),
)
def test_to_lon180(ds, expected):
    actual_ds = _to_lon180(ds)
    assert actual_ds == expected


__all__ = ['open_dataset', 'product_info', 'Sentinel1Meta', 'Sentinel1Dataset', 'RadarSat2Dataset', 'RadarSat2Meta', 'RcmMeta', 'RcmDataset', 'BaseDataset', 'BaseMeta']

from .xsar import *
from .xsar import __version__




try:
    # will fall back to repr if some modules are missing

    # make sure we are running from a notebook
    # if test fail, nothing will be imported, and that will save lot of importtime
    assert get_ipython() is not None
    import cartopy
    import holoviews as hv
    import geoviews as gv
    import geoviews.feature as gf
    import jinja2
    import geopandas as gpd
    import holoviews.ipython.display_hooks as display_hooks
    from shapely.geometry import Polygon, Point
    import pyproj
except (ModuleNotFoundError, AssertionError, NameError) as e:
    pass

def repr_mimebundle_Sentinel1Meta(self, include=None, exclude=None):
    """html output for notebook"""

    template = jinja2.Template(
        """
        <div align="left">
            <h5>{{ intro }}</h5>
            <table style="width:100%">
                <thead>
                    <tr>
                        <th colspan="2">{{ short_name }}</th>
                    </tr>
                </thead>
                <tbody>
                    <tr>
                        <td>
                            <table>
                                {% for key, value in properties.items() %}
                                 <tr>
                                     <th> {{ key }} </th>
                                     <td> {{ value }} </td>
                                 </tr>
                                {% endfor %}
                            </table>
                        </td>
                        <td>{{ location }}</td>
                    </tr>
                </tbody>
            </table>

        </div>

        """
    )

    crs = cartopy.crs.PlateCarree()

    world = gv.operation.resample_geometry(gf.land.geoms('10m')).opts(color='khaki', projection=crs, alpha=0.5)

    center = self.footprint.centroid
    xlim = (center.x - 20, center.x + 20)
    ylim = (center.y - 20, center.y + 20)

    if self.multidataset and len(self.subdatasets) == len(self._footprints):  # checks len because SAFEs like IW_SLC has only one footprint for 3 subdatasets
        dsid = [s.split(':')[2] for s in self.subdatasets]
        footprint = self._footprints
    else:
        dsid = [self.dsid]
        footprint = [self.footprint]

    footprints_df = gpd.GeoDataFrame(
        {
            'dsid': dsid,
            'geometry': footprint
        },
        crs='EPSG:4326'
    )

    opts = {
        'bokeh': dict(tools=['hover'])
    }

    footprint = gv.Polygons(footprints_df, label='footprint') \
        .opts(projection=crs, xlim=xlim, ylim=ylim, alpha=0.5) \
        .opts(**(opts.get(hv.Store.current_backend) or {}), backend=hv.Store.current_backend)

    orbit = gv.Points(
        self.orbit['geometry'].to_crs('EPSG:4326'), label='orbit'
    ).opts(projection=crs, xlim=xlim, ylim=ylim, alpha=0.5)\
        .opts(**(opts.get(hv.Store.current_backend) or {}), backend=hv.Store.current_backend)


    opts = {
        'bokeh': dict(width=400, height=400),
        'matplotlib': dict(fig_inches=5)
    }
    location = (world * footprint * orbit) \
        .opts(title='Map') \
        .opts(**(opts.get(hv.Store.current_backend) or {}), backend=hv.Store.current_backend) \

    data, metadata = display_hooks.render(location)

    properties = self.to_dict()
    properties['orbit_pass'] = self.orbit_pass
    if self.pixel_line_m is not None:
        properties['pixel size'] = "%.1f * %.1f meters (line * sample)" % (
            self.pixel_line_m, self.pixel_sample_m)
    properties['coverage'] = self.coverage
    properties['start_date'] = self.start_date
    properties['stop_date'] = self.stop_date
    if len(self.subdatasets) > 0:
        properties['subdatasets'] = "list of %d subdatasets" % len(self.subdatasets)
    properties = {k: v for k, v in properties.items() if v is not None}

    if self.multidataset:
        intro = "Multi (%d) dataset" % len(self.subdatasets)
    else:
        intro = "Single dataset"
        properties['dsid'] = self.dsid

    if 'text/html' in data:
        data['text/html'] = template.render(
            intro=intro,
            short_name=self.short_name,
            properties=properties,
            location=data['text/html']
        )

    return data, metadata


def repr_mimebundle_Sentinel1Dataset(self, include=None, exclude=None):

    template = jinja2.Template(
        """
        <div align="left">
            <h5>{{ intro }}</h5>
            <table style="width:100%">
                <thead>
                    <tr>
                        <th colspan="2">{{ short_name }}</th>
                    </tr>
                </thead>
                <tbody>
                    <tr>
                        <td>
                            <table>
                                {% for key, value in properties.items() %}
                                 <tr>
                                     <th> {{ key }} </th>
                                     <td> {{ value }} </td>
                                 </tr>
                                {% endfor %}
                            </table>
                        </td>
                        <td>{{ location }}</td>
                    </tr>
                </tbody>
            </table>

        </div>

        """
    )

    opts = {
        'bokeh': dict(fill_color='cyan')
    }
    grid = (
            hv.Path(Polygon(self._bbox_coords_ori)).opts(color='blue') \
            * hv.Polygons(Polygon(self._bbox_coords)) \
            .opts(color='blue') \
            .opts(**(opts.get(hv.Store.current_backend) or {}), backend=hv.Store.current_backend)
    ).opts(xlabel='line', ylabel='sample')

    data, metadata = display_hooks.render(grid)
    properties = {}
    if self.pixel_line_m is not None:
        properties['pixel size'] = "%.1f * %.1f meters (line * sample)" % (
            self.pixel_line_m, self.pixel_sample_m)
    properties['coverage'] = self.coverage
    properties = {k: v for k, v in properties.items() if v is not None}

    if self.sliced:
        intro = "dataset slice"
    else:
        intro = "full dataset coverage"

    if 'text/html' in data:
        data['text/html'] = template.render(
            intro=intro,
            short_name=self.sar_meta.short_name,
            properties=properties,
            location=data['text/html']
        )

    return data, metadata


repr_mimebundle_wrapper = {
    'Sentinel1Meta': repr_mimebundle_Sentinel1Meta,
    'Sentinel1Dataset': repr_mimebundle_Sentinel1Dataset,
}


def repr_mimebundle(obj, include=None, exclude=None):
    try:
        return repr_mimebundle_wrapper[type(obj).__name__](obj, include=include, exclude=exclude)
    except Exception as e:
        return {'text': repr(obj)}



from setuptools import setup, find_packages
import glob

setup(
    name='xsar',
    package_dir={'': 'src'},
    packages=find_packages('src'),
    scripts=glob.glob('src/scripts/*.py'),
    url='https://github.com/umr-lops/xsar',
    use_scm_version=True,
    setup_requires=['setuptools_scm'],
    include_package_data=True,
    install_requires=[
        'GDAL',
        'dask[array]',
        'distributed',
        'xarray',
        'affine',
        'rasterio',
        'cartopy',
        'fiona',
        'pyproj',
        'xarray-datatree>=0.0.9',
        'numpy',
        'scipy',
        'shapely',
        'geopandas',
        'fsspec',
        'aiohttp',
        'pytz',
        'psutil',
        'jinja2',
        'rioxarray',
        'lxml'
    ],
    extras_require={
        "RS2": ["xradarsat2"],
        "RCM": ["xarray-safe-rcm"],
        "S1": ["xarray-safe-s1"]
    },
    entry_points={
        "xarray.backends": ["xsar=xsar.xarray_backends:XsarXarrayBackend"]
    },
    license='MIT',
    author='Olivier Archer, Alexandre Levieux, Antoine Grouazel',
    author_email='Olivier.Archer@ifremer.fr, Alexandre.Levieux@gmail.com, Antoine.Grouazel@ifremer.fr',
    description='xarray L1 SAR mapper',
    summary='Python xarray library to use Level-1 GRD SAR products',
    long_description_content_type='text/x-rst',
    long_description = 'Python xarray library to use Level-1 GRD Synthetic Aperture Radar products'
)
